﻿using System.ServiceModel;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Security;
using DevExpress.ExpressApp.Security.ClientServer;
using DevExpress.ExpressApp.Security.ClientServer.Wcf;
using DevExpress.ExpressApp.Security.Strategy;
using DevExpress.ExpressApp.Xpo;
using Xafari;
using Xafari.AppModules;
using Xafari.Security;
using Xafari.Security.DC;

namespace test_xafarimidtierserver.App
{
	// See http://galaktikasoft.com/documentation/xafari/appmodule.html
	public sealed partial class test_xafarimidtierserverAppModule : AppModuleBase
	{
        private readonly SecurityStrategyComplex _securityStrategyComplex;
	    private ServerSecurityClient _securityClient;
	    private WcfSecuredDataServerClient _clientDataServer;
        public test_xafarimidtierserverAppModule()
		{
			InitializeComponent();

		    var xafariAuthentication = new XafariAuthentication(typeof(IDCSecuritySystemUser), typeof(XafariAuthenticationLogonParameters));
		    xafariAuthentication.AuthenticationActiveDirectory.CreateUserAutomatically = true;
		    _securityStrategyComplex = new SecurityStrategyComplex(typeof(IDCSecuritySystemUser), typeof(IDCSecuritySystemRole), xafariAuthentication);


		    ConnectionString = "http://localhost:1451/DataServer";
		    _clientDataServer = new WcfSecuredDataServerClient(
		        WcfDataServerHelper.CreateDefaultBinding(),
		        new EndpointAddress(ConnectionString));
		    _securityClient =
		        new ServerSecurityClient(_clientDataServer, new ClientInfoFactory());
		    _securityClient.IsSupportChangePassword = true;
		    

		    Security = _securityClient;

            
        }

        public override void Setup(XafApplication application)
	    {
	        base.Setup(application);
	        application.Security = _securityStrategyComplex;
	    }
	    public SecurityStrategy GetSecurityStrategy()
	    {
	        return _securityStrategyComplex;
	    }
        public override void InitializeApplication(XafApplication application)
		{
			base.InitializeApplication(application);

			// add here platform-agnostic code to initialize application instance

			application.LinkNewObjectToParentImmediately = false;
			if (System.Diagnostics.Debugger.IsAttached && application.CheckCompatibilityType == CheckCompatibilityType.DatabaseSchema)
				application.DatabaseUpdateMode = DatabaseUpdateMode.UpdateDatabaseAlways;

			application.CustomizeLanguagesList += this.test_xafarimidtierserverAppModule_CustomizeLanguagesList;
			application.Xafari().CustomizeApplicationContext += this.test_xafarimidtierserverAppModule_CustomizeApplicationContext;
            application.DatabaseVersionMismatch += application_DatabaseVersionMismatch;
        }

        void application_DatabaseVersionMismatch(object sender, DatabaseVersionMismatchEventArgs e)
	    {
	        e.Updater.Update();
	        e.Handled = true;
	    }

	    void test_xafarimidtierserverAppModule_CustomizeApplicationContext(object sender, Xafari.Helpers.CustomizeApplicationContextEventArgs args)
		{
			// add here additional application context
			args.Context.Add(XafariApplicationContext.DC);
	    }

		// override this method to customize ObjectSpace providers
		protected override void CreateObjectSpaceProviders(CreateCustomObjectSpaceProviderEventArgs args)
		{
			base.CreateObjectSpaceProviders(args);
		    args.ObjectSpaceProvider =
		        new DataServerObjectSpaceProvider(_clientDataServer, _securityClient);
        }

		private void test_xafarimidtierserverAppModule_CustomizeLanguagesList(object sender, CustomizeLanguagesListEventArgs e)
		{
			string userLanguageName = System.Threading.Thread.CurrentThread.CurrentUICulture.Name;
			if (userLanguageName != "en-US" && e.Languages.IndexOf(userLanguageName) == -1)
			{
				e.Languages.Add(userLanguageName);
			}
		}
	}
}
